@@ -2,10 +2,10 @@ package DBIx::TextIndex::TermDocsCache;
 
 use strict;
 
-our $VERSION = '0.15';
+our $VERSION = '0.17';
 
 use Bit::Vector;
-use DBIx::TextIndex;
+#use DBIx::TextIndex;
 
 sub new {
     my $pkg = shift;
@@ -37,8 +37,16 @@ sub max_indexed_id {
 
 sub flush {
     my $self = shift;
-    $self->{TERM_DOCS} = ();
-    $self->{DOCFREQ_T} = ();
+    delete($self->{TERM_DOCS});
+    delete($self->{DOCFREQ_T});
+}
+
+sub term_docs_hashref {
+    my $self = shift;
+    my ($fno, $term) = @_;
+    $self->_fetch_term_docs($fno, $term) unless exists $self->{TERM_DOCS}->[$fno]->{$term};
+    return DBIx::TextIndex::term_docs_hashref($self->{TERM_DOCS}->[$fno]->{$term});
+
 }
 
 sub term_docs_arrayref {
@@ -63,6 +71,14 @@ sub vector {
     return $vector;
 }
 
+# Same as docfreq_t
+sub f_t {
+    my $self = shift;
+    my ($fno, $term) = @_;
+    $self->_fetch_term_docs($fno, $term) unless exists $self->{DOCFREQ_T}->[$fno]->{$term};
+    return $self->{DOCFREQ_T}->[$fno]->{$term};
+}
+
 sub docfreq_t {
     my $self = shift;
     my ($fno, $term) = @_;
@@ -79,3 +95,27 @@ sub _fetch_term_docs {
     ($self->{DOCFREQ_T}->[$fno]->{$term}, $self->{TERM_DOCS}->[$fno]->{$term})
 	= $self->{DBH}->selectrow_array($sql, undef, $term);
 }
+
+sub _term_docs {
+    my $self = shift;
+    my ($fno, $term) = @_;
+ 
+    my $sql = $self->db_fetch_term_docs($self->{INVERTED_TABLES}->[$fno]);
+    my $sth = $self->{DBH}->prepare($sql);
+    $sth->execute($term);
+
+    my $docs;
+    $sth->bind_col(1, \$docs);
+    $sth->fetch;
+    return $docs;
+}
+
+sub _docfreq_t {
+    my $self = shift;
+    my ($fno, $term) = @_;
+    my $sql = $self->db_docfreq_t($self->{INVERTED_TABLES}->[$fno]);
+    my ($docfreq_t) =
+	$self->{DBH}->selectrow_array($sql, undef, $term);
+    return $docfreq_t;
+}
+
